﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using HeuristicLab.Algorithms.GrammaticalOptimization;
using HeuristicLab.Problems.GrammaticalOptimization;

namespace Main {
  class Program {
    static void Main(string[] args) {
      int maxIterations = 10000000;
      int iterations = 0;
      var sw = new Stopwatch();
      double bestQuality = 0;
      string bestSentence = "";

      var rs = new ExhaustiveDepthFirstSearch(17);

      rs.FoundNewBestSolution += (sentence, quality) => {
        bestQuality = quality;
        bestSentence = sentence;
        Console.WriteLine("{0,10} {1,10:F5} {2,10:F5} {3}", iterations, bestQuality, quality, sentence);
      };
      rs.SolutionEvaluated += (sentence, quality) => {
        iterations++;
        if (iterations % 10000 == 0) {
          Console.WriteLine("{0,10} {1,10:F5} {2,10:F5} {3}", iterations, bestQuality, quality, sentence);
        }
      };


      sw.Start();

      rs.Run(new SymbolicRegressionPoly10Problem(), maxIterations);

      sw.Stop();

      Console.WriteLine("{0,10} Best soultion: {1,10:F5} {2}", iterations, bestQuality, bestSentence);
      Console.WriteLine("{0:F2} sec {1,10:F1} sols/sec {2,10:F1} ns/sol",
        sw.Elapsed.TotalSeconds,
        maxIterations / (double)sw.Elapsed.TotalSeconds,
        (double)sw.ElapsedMilliseconds * 1000 / maxIterations);
    }
  }
}
